@@ -52,10 +52,8 @@ private Q_SLOTS:
     void cleanup();
     void testAxis_data();
     void testAxis();
-    void testDoubleClickOnAllDesktops_data();
     void testDoubleClickOnAllDesktops();
     void testDoubleClickClose();
-    void testDoubleTap_data();
     void testDoubleTap();
     void testHover();
     void testPressToMove_data();
@@ -72,7 +70,7 @@ private Q_SLOTS:
     void testTooltipDoesntEatKeyEvents();
 
 private:
-    std::tuple<Window *, std::unique_ptr<KWayland::Client::Surface>, Test::XdgToplevel *> showWindow();
+    std::tuple<Window *, std::unique_ptr<KWayland::Client::Surface>, std::unique_ptr<Test::XdgToplevel>, std::unique_ptr<Test::XdgToplevelDecorationV1>> showWindow();
 };
 
 #define MOTION(target) Test::pointerMotion(target, timestamp++)
@@ -81,23 +79,23 @@ private:
 
 #define RELEASE Test::pointerButtonReleased(BTN_LEFT, timestamp++)
 
-std::tuple<Window *, std::unique_ptr<KWayland::Client::Surface>, Test::XdgToplevel *> DecorationInputTest::showWindow()
+std::tuple<Window *, std::unique_ptr<KWayland::Client::Surface>, std::unique_ptr<Test::XdgToplevel>, std::unique_ptr<Test::XdgToplevelDecorationV1>> DecorationInputTest::showWindow()
 {
 #define VERIFY(statement)                                                 \
     if (!QTest::qVerify((statement), #statement, "", __FILE__, __LINE__)) \
-        return {nullptr, nullptr, nullptr};
+        return {nullptr, nullptr, nullptr, nullptr};
 #define COMPARE(actual, expected)                                                   \
     if (!QTest::qCompare(actual, expected, #actual, #expected, __FILE__, __LINE__)) \
-        return {nullptr, nullptr, nullptr};
+        return {nullptr, nullptr, nullptr, nullptr};
 
     std::unique_ptr<KWayland::Client::Surface> surface{Test::createSurface()};
     VERIFY(surface.get());
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), Test::CreationSetup::CreateOnly, surface.get());
-    VERIFY(shellSurface);
-    Test::XdgToplevelDecorationV1 *decoration = Test::createXdgToplevelDecorationV1(shellSurface, shellSurface);
-    VERIFY(decoration);
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get(), Test::CreationSetup::CreateOnly);
+    VERIFY(shellSurface.get());
+    std::unique_ptr<Test::XdgToplevelDecorationV1> decoration = Test::createXdgToplevelDecorationV1(shellSurface.get());
+    VERIFY(decoration.get());
 
-    QSignalSpy decorationConfigureRequestedSpy(decoration, &Test::XdgToplevelDecorationV1::configureRequested);
+    QSignalSpy decorationConfigureRequestedSpy(decoration.get(), &Test::XdgToplevelDecorationV1::configureRequested);
     QSignalSpy surfaceConfigureRequestedSpy(shellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
 
     decoration->set_mode(Test::XdgToplevelDecorationV1::mode_server_side);
@@ -114,7 +112,7 @@ std::tuple<Window *, std::unique_ptr<KWayland::Client::Surface>, Test::XdgToplev
 #undef VERIFY
 #undef COMPARE
 
-    return {window, std::move(surface), shellSurface};
+    return {window, std::move(surface), std::move(shellSurface), std::move(decoration)};
 }
 
 void DecorationInputTest::initTestCase()
@@ -174,7 +172,7 @@ void DecorationInputTest::testAxis()
 {
     static constexpr double oneTick = 15;
 
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -211,16 +209,6 @@ void DecorationInputTest::testAxis()
     QVERIFY(!window->keepAbove());
 }
 
-void DecorationInputTest::testDoubleClickOnAllDesktops_data()
-{
-    QTest::addColumn<QPoint>("decoPoint");
-    QTest::addColumn<Qt::WindowFrameSection>("expectedSection");
-
-    QTest::newRow("topLeft") << QPoint(0, 0) << Qt::TopLeftSection;
-    QTest::newRow("top") << QPoint(250, 0) << Qt::TopSection;
-    QTest::newRow("topRight") << QPoint(499, 0) << Qt::TopRightSection;
-}
-
 void KWin::DecorationInputTest::testDoubleClickOnAllDesktops()
 {
     KConfigGroup group = kwinApp()->config()->group(QStringLiteral("Windows"));
@@ -228,7 +216,7 @@ void KWin::DecorationInputTest::testDoubleClickOnAllDesktops()
     group.sync();
     workspace()->slotReconfigure();
 
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -249,21 +237,6 @@ void KWin::DecorationInputTest::testDoubleClickOnAllDesktops()
     PRESS;
     RELEASE;
     QVERIFY(!window->isOnAllDesktops());
-
-    // test top most deco pixel, BUG: 362860
-    window->move(QPoint(0, 0));
-    QFETCH(QPoint, decoPoint);
-    MOTION(decoPoint);
-    QVERIFY(input()->pointer()->decoration());
-    QCOMPARE(input()->pointer()->decoration()->window(), window);
-    QTEST(input()->pointer()->decoration()->decoration()->sectionUnderMouse(), "expectedSection");
-    // double click
-    PRESS;
-    RELEASE;
-    QVERIFY(!window->isOnAllDesktops());
-    PRESS;
-    RELEASE;
-    QVERIFY(window->isOnAllDesktops());
 }
 
 void DecorationInputTest::testDoubleClickClose()
@@ -274,13 +247,13 @@ void DecorationInputTest::testDoubleClickClose()
     group.sync();
     workspace()->slotReconfigure();
 
-    auto [window, surface, shellSurface] = showWindow();
+    auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     quint32 timestamp = 1;
     MOTION(QPoint(window->frameGeometry().center().x(), window->frameMargins().top() / 2.0));
 
-    connect(shellSurface, &Test::XdgToplevel::closeRequested, this, [&surface = surface]() {
+    connect(shellSurface.get(), &Test::XdgToplevel::closeRequested, this, [&surface = surface]() {
         surface.reset();
     });
 
@@ -297,16 +270,6 @@ void DecorationInputTest::testDoubleClickClose()
     window->unref();
 }
 
-void DecorationInputTest::testDoubleTap_data()
-{
-    QTest::addColumn<QPoint>("decoPoint");
-    QTest::addColumn<Qt::WindowFrameSection>("expectedSection");
-
-    QTest::newRow("topLeft") << QPoint(10, 10) << Qt::TopLeftSection;
-    QTest::newRow("top") << QPoint(260, 10) << Qt::TopSection;
-    QTest::newRow("topRight") << QPoint(509, 10) << Qt::TopRightSection;
-}
-
 void KWin::DecorationInputTest::testDoubleTap()
 {
     KConfigGroup group = kwinApp()->config()->group(QStringLiteral("Windows"));
@@ -314,7 +277,7 @@ void KWin::DecorationInputTest::testDoubleTap()
     group.sync();
     workspace()->slotReconfigure();
 
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -335,28 +298,11 @@ void KWin::DecorationInputTest::testDoubleTap()
     Test::touchDown(0, tapPoint, timestamp++);
     Test::touchUp(0, timestamp++);
     QVERIFY(!window->isOnAllDesktops());
-
-    // test top most deco pixel, BUG: 362860
-    //
-    // Not directly at (0, 0), otherwise ScreenEdgeInputFilter catches
-    // event before DecorationEventFilter.
-    window->move(QPoint(10, 10));
-    QFETCH(QPoint, decoPoint);
-    // double click
-    Test::touchDown(0, decoPoint, timestamp++);
-    QVERIFY(input()->touch()->decoration());
-    QCOMPARE(input()->touch()->decoration()->window(), window);
-    QTEST(input()->touch()->decoration()->decoration()->sectionUnderMouse(), "expectedSection");
-    Test::touchUp(0, timestamp++);
-    QVERIFY(!window->isOnAllDesktops());
-    Test::touchDown(0, decoPoint, timestamp++);
-    Test::touchUp(0, timestamp++);
-    QVERIFY(window->isOnAllDesktops());
 }
 
 void DecorationInputTest::testHover()
 {
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -415,7 +361,7 @@ void DecorationInputTest::testPressToMove_data()
 
 void DecorationInputTest::testPressToMove()
 {
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -472,7 +418,7 @@ void DecorationInputTest::testTapToMove_data()
 
 void DecorationInputTest::testTapToMove()
 {
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -536,7 +482,7 @@ void DecorationInputTest::testResizeOutsideWindow()
     workspace()->slotReconfigure();
 
     // now create window
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -630,7 +576,7 @@ void DecorationInputTest::testModifierClickUnrestrictedMove()
     QCOMPARE(options->commandAll3(), Options::MouseUnrestrictedMove);
 
     // create a window
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -692,7 +638,7 @@ void DecorationInputTest::testModifierScrollOpacity()
     group.sync();
     workspace()->slotReconfigure();
 
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -750,7 +696,7 @@ void DecorationInputTest::testTouchEvents()
 {
     // this test verifies that the decoration gets a hover leave event on touch release
     // see BUG 386231
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
@@ -796,7 +742,7 @@ void DecorationInputTest::testTooltipDoesntEatKeyEvents()
     QVERIFY(keyboard);
     QSignalSpy enteredSpy(keyboard, &KWayland::Client::Keyboard::entered);
 
-    const auto [window, surface, shellSurface] = showWindow();
+    const auto [window, surface, shellSurface, decoration] = showWindow();
     QVERIFY(window);
     QVERIFY(window->isDecorated());
     QVERIFY(!window->noBorder());
