@@ -38,17 +38,6 @@
 namespace KWin
 {
 
-static PlatformCursorImage loadReferenceThemeCursor_helper(const KXcursorTheme &theme,
-                                                           const QByteArray &name)
-{
-    const QList<KXcursorSprite> sprites = theme.shape(name);
-    if (sprites.isEmpty()) {
-        return PlatformCursorImage();
-    }
-
-    return PlatformCursorImage(sprites.constFirst().data(), sprites.constFirst().hotspot());
-}
-
 static PlatformCursorImage loadReferenceThemeCursor(const QByteArray &name)
 {
     const Cursor *pointerCursor = Cursors::self()->mouse();
@@ -58,20 +47,11 @@ static PlatformCursorImage loadReferenceThemeCursor(const QByteArray &name)
         return PlatformCursorImage();
     }
 
-    PlatformCursorImage platformCursorImage = loadReferenceThemeCursor_helper(theme, name);
-    if (!platformCursorImage.isNull()) {
-        return platformCursorImage;
-    }
+    ShapeCursorSource source;
+    source.setShape(name);
+    source.setTheme(theme);
 
-    const QList<QByteArray> alternativeNames = Cursor::cursorAlternativeNames(name);
-    for (const QByteArray &alternativeName : alternativeNames) {
-        platformCursorImage = loadReferenceThemeCursor_helper(theme, alternativeName);
-        if (!platformCursorImage.isNull()) {
-            break;
-        }
-    }
-
-    return platformCursorImage;
+    return PlatformCursorImage(source.image(), source.hotspot());
 }
 
 static PlatformCursorImage loadReferenceThemeCursor(const CursorShape &shape)
@@ -114,6 +94,8 @@ private Q_SLOTS:
     void testWindowUnderCursorWhileButtonPressed();
     void testConfineToScreenGeometry_data();
     void testConfineToScreenGeometry();
+    void testEdgeBarrier_data();
+    void testEdgeBarrier();
     void testResizeCursor_data();
     void testResizeCursor();
     void testMoveCursor();
@@ -165,6 +147,11 @@ void PointerInputTest::init()
     m_compositor = Test::waylandCompositor();
     m_seat = Test::waylandSeat();
 
+    auto group = kwinApp()->config()->group(QStringLiteral("EdgeBarrier"));
+    group.writeEntry("EdgeBarrier", 0);
+    group.writeEntry("CornerBarrier", false);
+    group.sync();
+    Workspace::self()->slotReconfigure();
     workspace()->setActiveOutput(QPoint(640, 512));
     input()->pointer()->warp(QPoint(640, 512));
 }
@@ -195,7 +182,7 @@ void PointerInputTest::testWarpingUpdatesFocus()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -240,7 +227,7 @@ void PointerInputTest::testWarpingGeneratesPointerMotion()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -318,7 +305,7 @@ void PointerInputTest::testUpdateFocusAfterScreenChange()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get(), QSize(1280, 1024));
     QVERIFY(windowAddedSpy.wait());
@@ -503,7 +490,7 @@ void PointerInputTest::testModifierClickUnrestrictedMove()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -568,10 +555,10 @@ void PointerInputTest::testModifierClickUnrestrictedFullscreenMove()
     // create a window
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     shellSurface->set_fullscreen(nullptr);
-    QSignalSpy toplevelConfigureRequestedSpy(shellSurface, &Test::XdgToplevel::configureRequested);
+    QSignalSpy toplevelConfigureRequestedSpy(shellSurface.get(), &Test::XdgToplevel::configureRequested);
     QSignalSpy surfaceConfigureRequestedSpy(shellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
     QVERIFY(surfaceConfigureRequestedSpy.wait());
     shellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
@@ -623,7 +610,7 @@ void PointerInputTest::testModifierClickUnrestrictedMoveGlobalShortcutsDisabled(
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -694,7 +681,7 @@ void PointerInputTest::testModifierScrollOpacity()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -752,7 +739,7 @@ void PointerInputTest::testModifierScrollOpacityGlobalShortcutsDisabled()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -799,7 +786,7 @@ void PointerInputTest::testScrollAction()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface1 = Test::createSurface();
     QVERIFY(surface1);
-    Test::XdgToplevel *shellSurface1 = Test::createXdgToplevelSurface(surface1.get(), surface1.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface1 = Test::createXdgToplevelSurface(surface1.get());
     QVERIFY(shellSurface1);
     render(surface1.get());
     QVERIFY(windowAddedSpy.wait());
@@ -807,7 +794,7 @@ void PointerInputTest::testScrollAction()
     QVERIFY(window1);
     std::unique_ptr<KWayland::Client::Surface> surface2 = Test::createSurface();
     QVERIFY(surface2);
-    Test::XdgToplevel *shellSurface2 = Test::createXdgToplevelSurface(surface2.get(), surface2.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface2 = Test::createXdgToplevelSurface(surface2.get());
     QVERIFY(shellSurface2);
     render(surface2.get());
     QVERIFY(windowAddedSpy.wait());
@@ -854,7 +841,7 @@ void PointerInputTest::testFocusFollowsMouse()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface1 = Test::createSurface();
     QVERIFY(surface1);
-    Test::XdgToplevel *shellSurface1 = Test::createXdgToplevelSurface(surface1.get(), surface1.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface1 = Test::createXdgToplevelSurface(surface1.get());
     QVERIFY(shellSurface1);
     render(surface1.get(), QSize(800, 800));
     QVERIFY(windowAddedSpy.wait());
@@ -862,7 +849,7 @@ void PointerInputTest::testFocusFollowsMouse()
     QVERIFY(window1);
     std::unique_ptr<KWayland::Client::Surface> surface2 = Test::createSurface();
     QVERIFY(surface2);
-    Test::XdgToplevel *shellSurface2 = Test::createXdgToplevelSurface(surface2.get(), surface2.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface2 = Test::createXdgToplevelSurface(surface2.get());
     QVERIFY(shellSurface2);
     render(surface2.get(), QSize(800, 800));
     QVERIFY(windowAddedSpy.wait());
@@ -936,7 +923,7 @@ void PointerInputTest::testMouseActionInactiveWindow()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface1 = Test::createSurface();
     QVERIFY(surface1);
-    Test::XdgToplevel *shellSurface1 = Test::createXdgToplevelSurface(surface1.get(), surface1.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface1 = Test::createXdgToplevelSurface(surface1.get());
     QVERIFY(shellSurface1);
     render(surface1.get(), QSize(800, 800));
     QVERIFY(windowAddedSpy.wait());
@@ -944,7 +931,7 @@ void PointerInputTest::testMouseActionInactiveWindow()
     QVERIFY(window1);
     std::unique_ptr<KWayland::Client::Surface> surface2 = Test::createSurface();
     QVERIFY(surface2);
-    Test::XdgToplevel *shellSurface2 = Test::createXdgToplevelSurface(surface2.get(), surface2.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface2 = Test::createXdgToplevelSurface(surface2.get());
     QVERIFY(shellSurface2);
     render(surface2.get(), QSize(800, 800));
     QVERIFY(windowAddedSpy.wait());
@@ -1021,7 +1008,7 @@ void PointerInputTest::testMouseActionActiveWindow()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface1 = Test::createSurface();
     QVERIFY(surface1);
-    Test::XdgToplevel *shellSurface1 = Test::createXdgToplevelSurface(surface1.get(), surface1.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface1 = Test::createXdgToplevelSurface(surface1.get());
     QVERIFY(shellSurface1);
     render(surface1.get(), QSize(800, 800));
     QVERIFY(windowAddedSpy.wait());
@@ -1030,7 +1017,7 @@ void PointerInputTest::testMouseActionActiveWindow()
     QSignalSpy window1DestroyedSpy(window1, &QObject::destroyed);
     std::unique_ptr<KWayland::Client::Surface> surface2 = Test::createSurface();
     QVERIFY(surface2);
-    Test::XdgToplevel *shellSurface2 = Test::createXdgToplevelSurface(surface2.get(), surface2.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface2 = Test::createXdgToplevelSurface(surface2.get());
     QVERIFY(shellSurface2);
     render(surface2.get(), QSize(800, 800));
     QVERIFY(windowAddedSpy.wait());
@@ -1097,7 +1084,7 @@ void PointerInputTest::testCursorImage()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -1227,7 +1214,7 @@ void PointerInputTest::testEffectOverrideCursorImage()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -1292,7 +1279,7 @@ void PointerInputTest::testPopup()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -1317,9 +1304,9 @@ void PointerInputTest::testPopup()
     positioner->set_gravity(Test::XdgPositioner::gravity_bottom_right);
     std::unique_ptr<KWayland::Client::Surface> popupSurface = Test::createSurface();
     QVERIFY(popupSurface);
-    Test::XdgPopup *popupShellSurface = Test::createXdgPopupSurface(popupSurface.get(), shellSurface->xdgSurface(), positioner.get());
+    std::unique_ptr<Test::XdgPopup> popupShellSurface = Test::createXdgPopupSurface(popupSurface.get(), shellSurface->xdgSurface(), positioner.get());
     QVERIFY(popupShellSurface);
-    QSignalSpy doneReceivedSpy(popupShellSurface, &Test::XdgPopup::doneReceived);
+    QSignalSpy doneReceivedSpy(popupShellSurface.get(), &Test::XdgPopup::doneReceived);
     popupShellSurface->grab(*Test::waylandSeat(), 0); // FIXME: Serial.
     render(popupSurface.get(), QSize(100, 50));
     QVERIFY(windowAddedSpy.wait());
@@ -1398,9 +1385,9 @@ void PointerInputTest::testDecoCancelsPopup()
     positioner->set_gravity(Test::XdgPositioner::gravity_bottom_right);
     std::unique_ptr<KWayland::Client::Surface> popupSurface = Test::createSurface();
     QVERIFY(popupSurface);
-    Test::XdgPopup *popupShellSurface = Test::createXdgPopupSurface(popupSurface.get(), shellSurface->xdgSurface(), positioner.get());
+    std::unique_ptr<Test::XdgPopup> popupShellSurface = Test::createXdgPopupSurface(popupSurface.get(), shellSurface->xdgSurface(), positioner.get());
     QVERIFY(popupShellSurface);
-    QSignalSpy doneReceivedSpy(popupShellSurface, &Test::XdgPopup::doneReceived);
+    QSignalSpy doneReceivedSpy(popupShellSurface.get(), &Test::XdgPopup::doneReceived);
     popupShellSurface->grab(*Test::waylandSeat(), 0); // FIXME: Serial.
     auto popupWindow = Test::renderAndWaitForShown(popupSurface.get(), QSize(100, 50), Qt::red);
     QVERIFY(popupWindow);
@@ -1435,7 +1422,7 @@ void PointerInputTest::testWindowUnderCursorWhileButtonPressed()
     QSignalSpy windowAddedSpy(workspace(), &Workspace::windowAdded);
     std::unique_ptr<KWayland::Client::Surface> surface = Test::createSurface();
     QVERIFY(surface);
-    Test::XdgToplevel *shellSurface = Test::createXdgToplevelSurface(surface.get(), surface.get());
+    std::unique_ptr<Test::XdgToplevel> shellSurface = Test::createXdgToplevelSurface(surface.get());
     QVERIFY(shellSurface);
     render(surface.get());
     QVERIFY(windowAddedSpy.wait());
@@ -1458,7 +1445,7 @@ void PointerInputTest::testWindowUnderCursorWhileButtonPressed()
     positioner->set_gravity(Test::XdgPositioner::gravity_bottom_right);
     std::unique_ptr<KWayland::Client::Surface> popupSurface = Test::createSurface();
     QVERIFY(popupSurface);
-    Test::XdgPopup *popupShellSurface = Test::createXdgPopupSurface(popupSurface.get(), shellSurface->xdgSurface(), positioner.get());
+    std::unique_ptr<Test::XdgPopup> popupShellSurface = Test::createXdgPopupSurface(popupSurface.get(), shellSurface->xdgSurface(), positioner.get());
     QVERIFY(popupShellSurface);
     render(popupSurface.get(), QSize(99, 49));
     QVERIFY(windowAddedSpy.wait());
@@ -1562,6 +1549,88 @@ void PointerInputTest::testConfineToScreenGeometry()
     QCOMPARE(Cursors::self()->mouse()->pos(), expectedPos);
 }
 
+void PointerInputTest::testEdgeBarrier_data()
+{
+    QTest::addColumn<QPoint>("startPos");
+    QTest::addColumn<QList<QPoint>>("movements");
+    QTest::addColumn<int>("targetOutputId");
+    QTest::addColumn<bool>("cornerBarrier");
+
+    // screen layout:
+    //
+    // +----------+----------+---------+
+    // |   left   |    top   |  right  |
+    // +----------+----------+---------+
+    //            |  bottom  |
+    //            +----------+
+    //
+    QTest::newRow("move right - barred") << QPoint(1270, 512) << QList<QPoint>{QPoint(20, 0)} << 0 << false;
+    QTest::newRow("move left - barred") << QPoint(1290, 512) << QList<QPoint>{QPoint(-20, 0)} << 1 << false;
+    QTest::newRow("move down - barred") << QPoint(1920, 1014) << QList<QPoint>{QPoint(0, 20)} << 1 << false;
+    QTest::newRow("move up - barred") << QPoint(1920, 1034) << QList<QPoint>{QPoint(0, -20)} << 3 << false;
+    QTest::newRow("move top-right - barred") << QPoint(2550, 1034) << QList<QPoint>{QPoint(20, -20)} << 3 << false;
+    QTest::newRow("move top-left - barred") << QPoint(1290, 1034) << QList<QPoint>{QPoint(-20, -20)} << 3 << false;
+    QTest::newRow("move bottom-right - barred") << QPoint(1270, 1014) << QList<QPoint>{QPoint(20, 20)} << 0 << false;
+    QTest::newRow("move bottom-left - barred") << QPoint(2570, 1014) << QList<QPoint>{QPoint(-20, 20)} << 2 << false;
+
+    QTest::newRow("move right - not barred") << QPoint(1270, 512) << QList<QPoint>{QPoint(100, 0)} << 1 << false;
+    QTest::newRow("move left - not barred") << QPoint(1290, 512) << QList<QPoint>{QPoint(-100, 0)} << 0 << false;
+    QTest::newRow("move down - not barred") << QPoint(1920, 1014) << QList<QPoint>{QPoint(0, 100)} << 3 << false;
+    QTest::newRow("move up - not barred") << QPoint(1920, 1034) << QList<QPoint>{QPoint(0, -100)} << 1 << false;
+    QTest::newRow("move top-right - not barred") << QPoint(2550, 1034) << QList<QPoint>{QPoint(100, -100)} << 2 << false;
+    QTest::newRow("move top-left - not barred") << QPoint(1290, 1034) << QList<QPoint>{QPoint(-100, -100)} << 0 << false;
+    QTest::newRow("move bottom-right - not barred") << QPoint(1270, 1014) << QList<QPoint>{QPoint(100, 100)} << 3 << false;
+    QTest::newRow("move bottom-left - not barred") << QPoint(2570, 1014) << QList<QPoint>{QPoint(-100, 100)} << 3 << false;
+
+    QTest::newRow("move cumulative") << QPoint(1279, 512) << QList<QPoint>{QPoint(24, 0), QPoint(24, 0)} << 1 << false;
+    QTest::newRow("move then idle") << QPoint(1279, 512) << QList<QPoint>{QPoint(24, 0), QPoint(0, 0), QPoint(0, 0), QPoint(3, 0)} << 0 << false;
+
+    QTest::newRow("move top-right - corner barrier") << QPoint(2550, 1034) << QList<QPoint>{QPoint(100, -100)} << 3 << true;
+    QTest::newRow("move top-left - corner barrier") << QPoint(1290, 1034) << QList<QPoint>{QPoint(-100, -100)} << 3 << true;
+    QTest::newRow("move bottom-right - corner barrier") << QPoint(1270, 1014) << QList<QPoint>{QPoint(100, 100)} << 0 << true;
+    QTest::newRow("move bottom-left - corner barrier") << QPoint(2570, 1014) << QList<QPoint>{QPoint(-100, 100)} << 2 << true;
+}
+
+void PointerInputTest::testEdgeBarrier()
+{
+    // setup screen layout
+    const QList<QRect> geometries{
+        QRect(0, 0, 1280, 1024),
+        QRect(1280, 0, 1280, 1024),
+        QRect(2560, 0, 1280, 1024),
+        QRect(1280, 1024, 1280, 1024)};
+    Test::setOutputConfig(geometries);
+
+    const auto outputs = workspace()->outputs();
+    QCOMPARE(outputs.count(), geometries.count());
+    QCOMPARE(outputs[0]->geometry(), geometries.at(0));
+    QCOMPARE(outputs[1]->geometry(), geometries.at(1));
+    QCOMPARE(outputs[2]->geometry(), geometries.at(2));
+    QCOMPARE(outputs[3]->geometry(), geometries.at(3));
+
+    QFETCH(QPoint, startPos);
+    input()->pointer()->warp(startPos);
+    quint32 timestamp = waylandServer()->seat()->timestamp().count() + 5000;
+    Test::pointerMotionRelative(QPoint(0, 0), timestamp);
+    timestamp += 1000;
+    QCOMPARE(Cursors::self()->mouse()->pos(), startPos);
+
+    auto group = kwinApp()->config()->group(QStringLiteral("EdgeBarrier"));
+    group.writeEntry("EdgeBarrier", 25);
+    QFETCH(bool, cornerBarrier);
+    group.writeEntry("CornerBarrier", cornerBarrier);
+    group.sync();
+    workspace()->slotReconfigure();
+
+    QFETCH(QList<QPoint>, movements);
+    for (const auto &movement : movements) {
+        Test::pointerMotionRelative(movement, timestamp);
+        timestamp += 1000;
+    }
+    QFETCH(int, targetOutputId);
+    QCOMPARE(workspace()->outputAt(Cursors::self()->mouse()->pos()), workspace()->outputs().at(targetOutputId));
+}
+
 void PointerInputTest::testResizeCursor_data()
 {
     QTest::addColumn<Qt::Edges>("edges");
