@@ -15,7 +15,7 @@
 namespace KWin
 {
 
-EffectsKeyboardInterceptionX11Filter::EffectsKeyboardInterceptionX11Filter(EffectsHandlerImpl *effects, X11Keyboard *keyboard)
+EffectsKeyboardInterceptionX11Filter::EffectsKeyboardInterceptionX11Filter(EffectsHandler *effects, X11Keyboard *keyboard)
     : X11EventFilter(QList<int>{XCB_KEY_PRESS, XCB_KEY_RELEASE})
     , m_effects(effects)
     , m_keyboard(keyboard)
