@@ -47,13 +47,14 @@ std::optional<RenderTimeSpan> CpuRenderTimeQuery::query()
 OutputFrame::OutputFrame(RenderLoop *loop, std::chrono::nanoseconds refreshDuration)
     : m_loop(loop)
     , m_refreshDuration(refreshDuration)
+    , m_targetPageflipTime(loop->nextPresentationTimestamp())
 {
 }
 
 OutputFrame::~OutputFrame()
 {
     Q_ASSERT(QThread::currentThread() == QCoreApplication::instance()->thread());
-    if (!m_presented) {
+    if (!m_presented && m_loop) {
         RenderLoopPrivate::get(m_loop)->notifyFrameDropped();
     }
 }
@@ -86,7 +87,9 @@ std::optional<std::chrono::nanoseconds> OutputFrame::queryRenderTime() const
 void OutputFrame::presented(std::chrono::nanoseconds timestamp, PresentationMode mode)
 {
     std::optional<std::chrono::nanoseconds> renderTime = queryRenderTime();
-    RenderLoopPrivate::get(m_loop)->notifyFrameCompleted(timestamp, renderTime, mode);
+    if (m_loop) {
+        RenderLoopPrivate::get(m_loop)->notifyFrameCompleted(timestamp, renderTime, mode);
+    }
     m_presented = true;
     for (const auto &feedback : m_feedbacks) {
         feedback->presented(m_refreshDuration, timestamp, mode);
@@ -128,6 +131,11 @@ void OutputFrame::addRenderTimeQuery(std::unique_ptr<RenderTimeQuery> &&query)
     m_renderTimeQueries.push_back(std::move(query));
 }
 
+std::chrono::steady_clock::time_point OutputFrame::targetPageflipTime() const
+{
+    return m_targetPageflipTime;
+}
+
 RenderBackend::RenderBackend(QObject *parent)
     : QObject(parent)
 {
