@@ -17,6 +17,7 @@ class RenderLoopPrivate;
 class SurfaceItem;
 class Item;
 class Output;
+class RenderLayer;
 
 /**
  * The RenderLoop class represents the compositing scheduler on a particular output.
@@ -75,7 +76,7 @@ public:
     /**
      * Schedules a compositing cycle at the next available moment.
      */
-    void scheduleRepaint(Item *item = nullptr);
+    void scheduleRepaint(Item *item = nullptr, RenderLayer *layer = nullptr);
 
     /**
      * Returns the timestamp of the last frame that has been presented on the screen.
