@@ -8,6 +8,8 @@
 #include <KDecoration2/DecorationButton>
 #include <QAbstractListModel>
 
+#include <QQmlEngine>
+
 namespace KDecoration2
 {
 
@@ -18,6 +20,7 @@ class PreviewBridge;
 class ButtonsModel : public QAbstractListModel
 {
     Q_OBJECT
+    QML_ELEMENT
 public:
     explicit ButtonsModel(const QList<DecorationButtonType> &buttons, QObject *parent = nullptr);
     explicit ButtonsModel(QObject *parent = nullptr);
