@@ -19,53 +19,19 @@
 namespace KWin
 {
 
-void GLFramebuffer::initStatic()
-{
-    if (GLPlatform::instance()->isGLES()) {
-        s_supported = true;
-        s_supportsPackedDepthStencil = hasGLVersion(3, 0) || hasGLExtension(QByteArrayLiteral("GL_OES_packed_depth_stencil"));
-        s_supportsDepth24 = hasGLVersion(3, 0) || hasGLExtension(QByteArrayLiteral("GL_OES_depth24"));
-        s_blitSupported = hasGLVersion(3, 0);
-    } else {
-        s_supported = hasGLVersion(3, 0) || hasGLExtension(QByteArrayLiteral("GL_ARB_framebuffer_object")) || hasGLExtension(QByteArrayLiteral("GL_EXT_framebuffer_object"));
-        s_supportsPackedDepthStencil = hasGLVersion(3, 0) || hasGLExtension(QByteArrayLiteral("GL_ARB_framebuffer_object")) || hasGLExtension(QByteArrayLiteral("GL_EXT_packed_depth_stencil"));
-        s_blitSupported = hasGLVersion(3, 0) || hasGLExtension(QByteArrayLiteral("GL_ARB_framebuffer_object")) || hasGLExtension(QByteArrayLiteral("GL_EXT_framebuffer_blit"));
-    }
-}
-
-void GLFramebuffer::cleanup()
-{
-    Q_ASSERT(s_fbos.isEmpty());
-    s_supported = false;
-    s_blitSupported = false;
-}
-
-bool GLFramebuffer::blitSupported()
-{
-    return s_blitSupported;
-}
-
 GLFramebuffer *GLFramebuffer::currentFramebuffer()
 {
-    return s_fbos.isEmpty() ? nullptr : s_fbos.top();
+    return OpenGlContext::currentContext()->currentFramebuffer();
 }
 
 void GLFramebuffer::pushFramebuffer(GLFramebuffer *fbo)
 {
-    fbo->bind();
-    s_fbos.push(fbo);
+    OpenGlContext::currentContext()->pushFramebuffer(fbo);
 }
 
 GLFramebuffer *GLFramebuffer::popFramebuffer()
 {
-    GLFramebuffer *ret = s_fbos.pop();
-    if (!s_fbos.isEmpty()) {
-        s_fbos.top()->bind();
-    } else {
-        glBindFramebuffer(GL_FRAMEBUFFER, 0);
-    }
-
-    return ret;
+    return OpenGlContext::currentContext()->popFramebuffer();
 }
 
 GLFramebuffer::GLFramebuffer()
@@ -109,11 +75,6 @@ GLFramebuffer::GLFramebuffer(GLTexture *colorAttachment, Attachment attachment)
     : m_size(colorAttachment->size())
     , m_colorAttachment(colorAttachment)
 {
-    if (!s_supported) {
-        qCCritical(KWIN_OPENGL) << "Framebuffer objects aren't supported!";
-        return;
-    }
-
     GLuint prevFbo = 0;
     if (const GLFramebuffer *current = currentFramebuffer()) {
         prevFbo = current->handle();
@@ -184,9 +145,9 @@ void GLFramebuffer::initColorAttachment(GLTexture *colorAttachment)
 void GLFramebuffer::initDepthStencilAttachment()
 {
     GLuint buffer = 0;
-
+    const auto context = OpenGlContext::currentContext();
     // Try to attach a depth/stencil combined attachment.
-    if (s_supportsPackedDepthStencil) {
+    if (context->supportsBlits()) {
         glGenRenderbuffers(1, &buffer);
         glBindRenderbuffer(GL_RENDERBUFFER, buffer);
         glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH24_STENCIL8, m_size.width(), m_size.height());
@@ -204,8 +165,8 @@ void GLFramebuffer::initDepthStencilAttachment()
 
     // Try to attach a depth attachment separately.
     GLenum depthFormat;
-    if (GLPlatform::instance()->isGLES()) {
-        if (s_supportsDepth24) {
+    if (context->isOpenGLES()) {
+        if (context->supportsGLES24BitDepthBuffers()) {
             depthFormat = GL_DEPTH_COMPONENT24;
         } else {
             depthFormat = GL_DEPTH_COMPONENT16;
@@ -226,7 +187,7 @@ void GLFramebuffer::initDepthStencilAttachment()
 
     // Try to attach a stencil attachment separately.
     GLenum stencilFormat;
-    if (GLPlatform::instance()->isGLES()) {
+    if (context->isOpenGLES()) {
         stencilFormat = GL_STENCIL_INDEX8;
     } else {
         stencilFormat = GL_STENCIL_INDEX;
@@ -287,7 +248,7 @@ bool GLFramebuffer::blitFromRenderTarget(const RenderTarget &sourceRenderTarget,
     const bool normal = transform == OutputTransform::Normal;
     const bool mirrorX = transform == OutputTransform::FlipX;
     const bool mirrorY = transform == OutputTransform::FlipY;
-    if ((normal || mirrorX || mirrorY) && blitSupported()) {
+    if ((normal || mirrorX || mirrorY) && OpenGlContext::currentContext()->supportsBlits()) {
         // either no transformation or flipping only
         blitFromFramebuffer(sourceViewport.mapToRenderTarget(source), destination, GL_LINEAR, mirrorX, mirrorY);
         return true;
