@@ -95,11 +95,12 @@ const QByteArray GLShader::prepareSource(GLenum shaderType, const QByteArray &so
 {
     // Prepare the source code
     QByteArray ba;
-    if (GLPlatform::instance()->isGLES() && GLPlatform::instance()->glslVersion() < Version(3, 0)) {
+    const auto context = OpenGlContext::currentContext();
+    if (context->isOpenGLES() && context->glslVersion() < Version(3, 0)) {
         ba.append("precision highp float;\n");
     }
     ba.append(source);
-    if (GLPlatform::instance()->isGLES() && GLPlatform::instance()->glslVersion() >= Version(3, 0)) {
+    if (context->isOpenGLES() && context->glslVersion() >= Version(3, 0)) {
         ba.replace("#version 140", "#version 300 es\n\nprecision highp float;\n");
     }
 
@@ -187,7 +188,8 @@ void GLShader::bindAttributeLocation(const char *name, int index)
 
 void GLShader::bindFragDataLocation(const char *name, int index)
 {
-    if (!GLPlatform::instance()->isGLES() && (hasGLVersion(3, 0) || hasGLExtension(QByteArrayLiteral("GL_EXT_gpu_shader4")))) {
+    const auto context = OpenGlContext::currentContext();
+    if (!context->isOpenGLES() && (context->hasVersion(Version(3, 0)) || context->hasOpenglExtension(QByteArrayLiteral("GL_EXT_gpu_shader4")))) {
         glBindFragDataLocation(m_program, index, name);
     }
 }
@@ -432,7 +434,7 @@ QMatrix4x4 GLShader::getUniformMatrix4x4(const char *name)
     int location = uniformLocation(name);
     if (location >= 0) {
         GLfloat m[16];
-        glGetnUniformfv(m_program, location, sizeof(m), m);
+        OpenGlContext::currentContext()->glGetnUniformfv(m_program, location, sizeof(m), m);
         QMatrix4x4 matrix(m[0], m[4], m[8], m[12],
                           m[1], m[5], m[9], m[13],
                           m[2], m[6], m[10], m[14],
