@@ -300,8 +300,8 @@ bool ButtonRebindsFilter::sendKeySequence(const QKeySequence &keys, bool pressed
         }
     }
 
-    QKeyEvent ev(QEvent::KeyPress, keys[0] & ~Qt::KeyboardModifierMask, Qt::NoModifier);
-    QList<xkb_keysym_t> syms(QXkbCommon::toKeysym(&ev));
+    QList<xkb_keysym_t> syms = KWin::Xkb::keysymsFromQtKey(keys[0]);
+
     // Use keysyms from the keypad if and only if KeypadModifier is set
     syms.erase(std::remove_if(syms.begin(), syms.end(), [keys](int sym) {
         bool onKeyPad = sym >= XKB_KEY_KP_Space && sym <= XKB_KEY_KP_Equal;
@@ -310,7 +310,9 @@ bool ButtonRebindsFilter::sendKeySequence(const QKeySequence &keys, bool pressed
         } else {
             return onKeyPad;
         }
-    }), syms.end());
+    }),
+               syms.end());
+
     if (syms.empty()) {
         qCWarning(KWIN_BUTTONREBINDS) << "Could not convert" << keys << "to keysym";
         return false;
