@@ -8,11 +8,8 @@
 
 #pragma once
 
-#include "config-kwin.h"
-
 #include "wayland/screencast_v1.h"
 
-#include <QDateTime>
 #include <QHash>
 #include <QObject>
 #include <QRegion>
@@ -30,9 +27,9 @@ namespace KWin
 {
 
 class Cursor;
-class ScreenCastDmaBufTexture;
 class GLTexture;
 class PipeWireCore;
+class ScreenCastBuffer;
 class ScreenCastSource;
 
 struct ScreenCastDmaBufTextureParams
@@ -51,6 +48,14 @@ public:
     explicit ScreenCastStream(ScreenCastSource *source, std::shared_ptr<PipeWireCore> pwCore, QObject *parent);
     ~ScreenCastStream();
 
+    enum class Content {
+        None,
+        Video = 0x1,
+        Cursor = 0x2,
+    };
+    Q_FLAG(Content)
+    Q_DECLARE_FLAGS(Contents, Content)
+
     bool init();
     uint framerate();
     uint nodeId();
@@ -61,17 +66,12 @@ public:
 
     void close();
 
-    /**
-     * Renders @p frame into the current framebuffer into the stream
-     * @p timestamp
-     */
-    void recordFrame(const QRegion &damagedRegion);
+    void recordFrame(const QRegion &damage, Contents contents = Content::Video);
 
     void setCursorMode(ScreencastV1Interface::CursorMode mode, qreal scale, const QRectF &viewport);
 
 public Q_SLOTS:
     void invalidateCursor();
-    void recordCursor();
     bool includesCursor(Cursor *cursor) const;
 
 Q_SIGNALS:
@@ -83,29 +83,27 @@ private:
     void onStreamStateChanged(pw_stream_state old, pw_stream_state state, const char *error_message);
     void onStreamAddBuffer(pw_buffer *buffer);
     void onStreamRemoveBuffer(pw_buffer *buffer);
-    void onStreamRenegotiateFormat(uint64_t);
 
     bool createStream();
     QList<const spa_pod *> buildFormats(bool fixate, char buffer[2048]);
     void updateParams();
+    void resize(const QSize &resolution);
     void coreFailed(const QString &errorMessage);
-    void sendCursorData(Cursor *cursor, spa_meta_cursor *spa_cursor);
+    void addCursorMetadata(spa_buffer *spaBuffer, Cursor *cursor);
+    QRegion addCursorEmbedded(ScreenCastBuffer *buffer, Cursor *cursor);
     void addHeader(spa_buffer *spaBuffer);
     void corruptHeader(spa_buffer *spaBuffer);
     void addDamage(spa_buffer *spaBuffer, const QRegion &damagedRegion);
     void newStreamParams();
-    void enqueue(pw_buffer *buffer);
     spa_pod *buildFormat(struct spa_pod_builder *b, enum spa_video_format format, struct spa_rectangle *resolution,
                          struct spa_fraction *defaultFramerate, struct spa_fraction *minFramerate, struct spa_fraction *maxFramerate,
                          const QList<uint64_t> &modifiers, quint32 modifiersFlags);
 
     std::optional<ScreenCastDmaBufTextureParams> testCreateDmaBuf(const QSize &size, quint32 format, const QList<uint64_t> &modifiers);
-    std::shared_ptr<ScreenCastDmaBufTexture> createDmaBufTexture(const ScreenCastDmaBufTextureParams &params);
 
     std::shared_ptr<PipeWireCore> m_pwCore;
     std::unique_ptr<ScreenCastSource> m_source;
     struct pw_stream *m_pwStream = nullptr;
-    struct spa_source *m_pwRenegotiate = nullptr;
     spa_hook m_streamListener;
     pw_stream_events m_pwStreamEvents = {};
 
@@ -113,7 +111,6 @@ private:
 
     QSize m_resolution;
     bool m_closed = false;
-    bool m_streaming = false;
 
     spa_video_info_raw m_videoFormat;
     QString m_error;
@@ -134,16 +131,16 @@ private:
         QMetaObject::Connection positionChangedConnection = QMetaObject::Connection();
     } m_cursor;
 
-    QHash<struct pw_buffer *, std::shared_ptr<ScreenCastDmaBufTexture>> m_dmabufDataForPwBuffer;
-
     quint64 m_sequential = 0;
     bool m_hasDmaBuf = false;
-    bool m_waitForNewBuffers = false;
     quint32 m_drmFormat = 0;
 
     std::optional<std::chrono::steady_clock::time_point> m_lastSent;
-    QRegion m_pendingDamages;
+    QRegion m_pendingDamage;
     QTimer m_pendingFrame;
+    Contents m_pendingContents = Content::None;
 };
 
 } // namespace KWin
+
+Q_DECLARE_OPERATORS_FOR_FLAGS(KWin::ScreenCastStream::Contents)
