@@ -11,7 +11,7 @@
 */
 #include "pointer_input.h"
 
-#include <config-kwin.h>
+#include "config-kwin.h"
 
 #include "core/output.h"
 #include "cursorsource.h"
@@ -21,20 +21,24 @@
 #include "input_event_spy.h"
 #include "mousebuttons.h"
 #include "osd.h"
+#include "screenedge.h"
 #include "wayland/display.h"
 #include "wayland/pointer.h"
 #include "wayland/pointerconstraints_v1.h"
 #include "wayland/seat.h"
 #include "wayland/surface.h"
 #include "wayland_server.h"
+#include "window.h"
 #include "workspace.h"
-#include "x11window.h"
 // KDecoration
 #include <KDecoration2/Decoration>
 // screenlocker
 #if KWIN_BUILD_SCREENLOCKER
 #include <KScreenLocker/KsldApp>
 #endif
+#if KWIN_BUILD_X11
+#include "x11window.h"
+#endif
 
 #include <KLocalizedString>
 
@@ -243,7 +247,7 @@ void PointerInputRedirection::processMotionInternal(const QPointF &pos, const QP
     }
 
     PositionUpdateBlocker blocker(this);
-    updatePosition(pos);
+    updatePosition(pos, time);
     MouseEvent event(QEvent::MouseMove, m_pos, Qt::NoButton, m_qtButtons,
                      input()->keyboardModifiers(), time,
                      delta, deltaNonAccelerated, device, type == MotionType::Warp);
@@ -739,8 +743,89 @@ QPointF PointerInputRedirection::applyPointerConfinement(const QPointF &pos) con
     return m_pos;
 }
 
-void PointerInputRedirection::updatePosition(const QPointF &pos)
+PointerInputRedirection::EdgeBarrierType PointerInputRedirection::edgeBarrierType(const QPointF &pos, const QRectF &lastOutputGeometry) const
 {
+    constexpr qreal cornerThreshold = 15;
+    const auto moveResizeWindow = workspace()->moveResizeWindow();
+    const bool onCorner = (pos - lastOutputGeometry.topLeft()).manhattanLength() <= cornerThreshold
+        || (pos - lastOutputGeometry.bottomLeft()).manhattanLength() <= cornerThreshold
+        || (pos - lastOutputGeometry.topRight()).manhattanLength() <= cornerThreshold
+        || (pos - lastOutputGeometry.bottomRight()).manhattanLength() <= cornerThreshold;
+    if (moveResizeWindow && moveResizeWindow->isInteractiveMove()) {
+        return EdgeBarrierType::WindowMoveBarrier;
+    } else if (moveResizeWindow && moveResizeWindow->isInteractiveResize()) {
+        return EdgeBarrierType::WindowResizeBarrier;
+    } else if (options->cornerBarrier() && onCorner) {
+        return EdgeBarrierType::CornerBarrier;
+    } else if (workspace()->screenEdges()->inApproachGeometry(pos.toPoint())) {
+        return EdgeBarrierType::EdgeElementBarrier;
+    } else {
+        return EdgeBarrierType::NormalBarrier;
+    }
+}
+
+qreal PointerInputRedirection::edgeBarrier(EdgeBarrierType type) const
+{
+    const auto barrierWidth = options->edgeBarrier();
+    switch (type) {
+    case EdgeBarrierType::WindowMoveBarrier:
+    case EdgeBarrierType::WindowResizeBarrier:
+        return 1.5 * barrierWidth;
+    case EdgeBarrierType::EdgeElementBarrier:
+        return 2 * barrierWidth;
+    case EdgeBarrierType::CornerBarrier:
+        return 2000;
+    case EdgeBarrierType::NormalBarrier:
+        return barrierWidth;
+    default:
+        Q_UNREACHABLE();
+        return 0;
+    }
+}
+
+QPointF PointerInputRedirection::applyEdgeBarrier(const QPointF &pos, const Output *currentOutput, std::chrono::microseconds time)
+{
+    // optimization to avoid looping over all outputs
+    if (exclusiveContains(currentOutput->geometry(), m_pos)) {
+        m_movementInEdgeBarrier = QPointF();
+        return pos;
+    }
+    const Output *lastOutput = workspace()->outputAt(m_pos);
+    QPointF newPos = confineToBoundingBox(pos, lastOutput->geometry());
+    const auto type = edgeBarrierType(newPos, lastOutput->geometry());
+    if (m_lastEdgeBarrierType != type) {
+        m_movementInEdgeBarrier = QPointF();
+    }
+    m_lastEdgeBarrierType = type;
+    const auto barrierWidth = edgeBarrier(type);
+    const qreal returnSpeed = barrierWidth / 10.0 /* px/s */ / 1000'000.0; // px/us
+    std::chrono::microseconds timeDiff(time - m_lastMoveTime);
+    qreal returnDistance = returnSpeed * timeDiff.count();
+
+    const auto euclideanLength = [](const QPointF &point) {
+        return std::sqrt(point.x() * point.x() + point.y() * point.y());
+    };
+    const auto shorten = [euclideanLength](const QPointF &point, const qreal distance) {
+        const qreal length = euclideanLength(point);
+        if (length <= distance) {
+            return QPointF();
+        }
+        return point * (1 - distance / length);
+    };
+
+    m_movementInEdgeBarrier += (pos - newPos);
+    m_movementInEdgeBarrier = shorten(m_movementInEdgeBarrier, returnDistance);
+
+    if (euclideanLength(m_movementInEdgeBarrier) > barrierWidth) {
+        newPos += shorten(m_movementInEdgeBarrier, barrierWidth);
+        m_movementInEdgeBarrier = QPointF();
+    }
+    return newPos;
+}
+
+void PointerInputRedirection::updatePosition(const QPointF &pos, std::chrono::microseconds time)
+{
+    m_lastMoveTime = time;
     if (m_locked) {
         // locked pointer should not move
         return;
@@ -748,6 +833,7 @@ void PointerInputRedirection::updatePosition(const QPointF &pos)
     // verify that at least one screen contains the pointer position
     const Output *currentOutput = workspace()->outputAt(pos);
     QPointF p = confineToBoundingBox(pos, currentOutput->geometry());
+    p = applyEdgeBarrier(p, currentOutput, time);
     p = applyPointerConfinement(p);
     if (p == m_pos) {
         // didn't change due to confinement
@@ -760,7 +846,7 @@ void PointerInputRedirection::updatePosition(const QPointF &pos)
 
     m_pos = p;
 
-    workspace()->setActiveCursorOutput(m_pos);
+    workspace()->setActiveOutput(m_pos);
     m_cursor->updateCursorOutputs(m_pos);
 
     Q_EMIT input()->globalPointerChanged(m_pos);
