@@ -11,6 +11,7 @@
 #include "effect/effect.h"
 #include "scene/cursoritem.h"
 #include "scene/itemrenderer.h"
+#include "scene/rootitem.h"
 
 namespace KWin
 {
@@ -18,7 +19,8 @@ namespace KWin
 CursorScene::CursorScene(std::unique_ptr<ItemRenderer> &&renderer)
     : Scene(std::move(renderer))
 {
-    m_rootItem = std::make_unique<CursorItem>(this);
+    m_rootItem = std::make_unique<RootItem>(this);
+    m_cursorItem = std::make_unique<CursorItem>(m_rootItem.get());
     setGeometry(m_rootItem->boundingRect().toRect());
     connect(m_rootItem.get(), &Item::boundingRectChanged, this, [this]() {
         setGeometry(m_rootItem->boundingRect().toRect());
@@ -55,10 +57,10 @@ void CursorScene::postPaint()
 
 void CursorScene::paint(const RenderTarget &renderTarget, const QRegion &region)
 {
-    RenderViewport viewport(QRectF(QPointF(), QSizeF(renderTarget.size()) / m_paintedOutput->scale()), m_paintedOutput->scale(), renderTarget);
+    RenderViewport viewport(QRectF(geometry().topLeft(), QSizeF(renderTarget.size()) / m_paintedOutput->scale()), m_paintedOutput->scale(), renderTarget);
     m_renderer->beginFrame(renderTarget, viewport);
     m_renderer->renderBackground(renderTarget, viewport, region);
-    m_renderer->renderItem(renderTarget, viewport, m_rootItem.get(), 0, region, WindowPaintData(viewport.projectionMatrix()));
+    m_renderer->renderItem(renderTarget, viewport, m_rootItem.get(), 0, region, WindowPaintData{});
     m_renderer->endFrame();
 }
 
