@@ -188,10 +188,14 @@ void Edge::reserveTouchCallBack(const TouchCallback &callback)
         })) {
         return;
     }
+    const bool wasTouch = activatesForTouchGesture();
     connect(callback.touchUpAction(), &QAction::destroyed, this, [this, callback]() {
         unreserveTouchCallBack(callback.touchUpAction());
     });
     m_touchCallbacks << callback;
+    if (wasTouch != activatesForTouchGesture()) {
+        Q_EMIT activatesForTouchGestureChanged();
+    }
     reserve();
 }
 
@@ -200,10 +204,16 @@ void Edge::unreserveTouchCallBack(QAction *action)
     auto it = std::find_if(m_touchCallbacks.begin(), m_touchCallbacks.end(), [action](const TouchCallback &c) {
         return c.touchUpAction() == action;
     });
-    if (it != m_touchCallbacks.end()) {
-        m_touchCallbacks.erase(it);
-        unreserve();
+    if (it == m_touchCallbacks.end()) {
+        return;
     }
+
+    const bool wasTouch = activatesForTouchGesture();
+    m_touchCallbacks.erase(it);
+    if (wasTouch != activatesForTouchGesture()) {
+        Q_EMIT activatesForTouchGestureChanged();
+    }
+    unreserve();
 }
 
 void Edge::unreserve()
@@ -797,8 +807,6 @@ ScreenEdges::ScreenEdges()
 {
     const int gridUnit = QFontMetrics(QFontDatabase::systemFont(QFontDatabase::GeneralFont)).boundingRect(QLatin1Char('M')).height();
     m_cornerOffset = 4 * gridUnit;
-
-    connect(workspace(), &Workspace::windowRemoved, this, &ScreenEdges::deleteEdgeForClient);
 }
 
 void ScreenEdges::init()
@@ -1340,6 +1348,10 @@ void ScreenEdges::unreserveTouch(ElectricBorder border, QAction *action)
 
 bool ScreenEdges::createEdgeForClient(Window *client, ElectricBorder border)
 {
+    if (client->isDeleted()) {
+        return false;
+    }
+
     int y = 0;
     int x = 0;
     int width = 0;
@@ -1391,10 +1403,12 @@ bool ScreenEdges::createEdgeForClient(Window *client, ElectricBorder border)
         return false;
     }
 
-    m_edges.push_back(createEdge(border, x, y, width, height, output, false));
-    Edge *edge = m_edges.back().get();
+    const auto &edge = m_edges.emplace_back(createEdge(border, x, y, width, height, output, false));
     edge->setClient(client);
     edge->reserve();
+    connect(client, &Window::closed, edge.get(), [this, client]() {
+        deleteEdgeForClient(client);
+    });
     return true;
 }
 
