@@ -10,7 +10,7 @@
 
 #pragma once
 
-#include "libkwineffects/kwinanimationeffect.h"
+#include "effect/animationeffect.h"
 
 #include <QJSEngine>
 #include <QJSValue>
@@ -18,6 +18,8 @@
 class KConfigLoader;
 class KPluginMetaData;
 
+class QAction;
+
 namespace KWin
 {
 
