@@ -50,6 +50,8 @@ class DrmLeaseManagerV1;
 class TearingControlManagerV1Interface;
 class XwaylandShellV1Interface;
 class OutputOrderV1Interface;
+class XdgDialogWmV1Interface;
+
 class Window;
 class Output;
 class XdgActivationV1Integration;
@@ -58,6 +60,8 @@ class XdgSurfaceWindow;
 class XdgToplevelWindow;
 class PresentationTime;
 class XXColorManagerV2;
+class LinuxDrmSyncObjV1Interface;
+class RenderBackend;
 
 class KWIN_EXPORT WaylandServer : public QObject
 {
@@ -123,10 +127,12 @@ public:
     {
         return m_keyboardShortcutsInhibitManager;
     }
+#if KWIN_BUILD_X11
     XwaylandShellV1Interface *xwaylandShell() const
     {
         return m_xwaylandShell;
     }
+#endif
 
     bool isKeyboardShortcutsInhibited() const;
 
@@ -160,11 +166,13 @@ public:
      */
     XdgExportedSurface *exportAsForeign(SurfaceInterface *surface);
 
+#if KWIN_BUILD_X11
     /**
      * @returns file descriptor for Xwayland to connect to.
      */
     int createXWaylandConnection();
     void destroyXWaylandConnection();
+#endif
 
     /**
      * @returns file descriptor to the input method server's socket.
@@ -226,6 +234,10 @@ public:
         return m_xdgActivationIntegration;
     }
 
+    LinuxDrmSyncObjV1Interface *linuxSyncObj() const;
+
+    void setRenderBackend(RenderBackend *backend);
+
 Q_SIGNALS:
     void windowAdded(KWin::Window *);
     void windowRemoved(KWin::Window *);
@@ -279,11 +291,14 @@ private:
     ClientConnection *m_screenLockerClientConnection = nullptr;
     XdgForeignV2Interface *m_XdgForeign = nullptr;
     XdgActivationV1Integration *m_xdgActivationIntegration = nullptr;
+#if KWIN_BUILD_X11
     XWaylandKeyboardGrabManagerV1Interface *m_xWaylandKeyboardGrabManager = nullptr;
+#endif
     ContentTypeManagerV1Interface *m_contentTypeManager = nullptr;
     TearingControlManagerV1Interface *m_tearingControlInterface = nullptr;
     XwaylandShellV1Interface *m_xwaylandShell = nullptr;
     PresentationTime *m_presentationTime = nullptr;
+    LinuxDrmSyncObjV1Interface *m_linuxDrmSyncObj = nullptr;
     QList<Window *> m_windows;
     InitializationFlags m_initFlags;
     QHash<Output *, OutputInterface *> m_waylandOutputs;
@@ -291,6 +306,7 @@ private:
     DrmLeaseManagerV1 *m_leaseManager = nullptr;
     OutputOrderV1Interface *m_outputOrder = nullptr;
     XXColorManagerV2 *m_xxColorManager = nullptr;
+    XdgDialogWmV1Interface *m_xdgDialogWm = nullptr;
     KWIN_SINGLETON(WaylandServer)
 };
 
