@@ -10,6 +10,8 @@
 */
 #include "xdgshellwindow.h"
 #include "core/output.h"
+#include "effect/globals.h"
+#include "utils/common.h"
 #if KWIN_BUILD_ACTIVITIES
 #include "activities.h"
 #endif
@@ -18,6 +20,7 @@
 #include "placement.h"
 #include "pointer_input.h"
 #include "tablet_input.h"
+#include "tiles/tilemanager.h"
 #include "touch_input.h"
 #include "utils/subsurfacemonitor.h"
 #include "virtualdesktops.h"
@@ -30,6 +33,7 @@
 #include "wayland/surface.h"
 #include "wayland/tablet_v2.h"
 #include "wayland/xdgdecoration_v1.h"
+#include "wayland/xdgdialog_v1.h"
 #include "wayland_server.h"
 #include "workspace.h"
 
@@ -90,7 +94,7 @@ XdgSurfaceWindow::~XdgSurfaceWindow()
 {
 }
 
-NET::WindowType XdgSurfaceWindow::windowType() const
+WindowType XdgSurfaceWindow::windowType() const
 {
     return m_windowType;
 }
@@ -213,12 +217,20 @@ void XdgSurfaceWindow::handleNextWindowGeometry()
         }
     }
 
-    if (!isInteractiveMoveResize()) {
-        // Both the compositor and the client can change the window geometry. If the client
-        // sets a new window geometry, the compositor's move-resize geometry will be invalid.
-        maybeUpdateMoveResizeGeometry(frameGeometry);
+    if (isInteractiveMove()) {
+        bool fullscreen = isFullScreen();
+        if (const auto configureEvent = static_cast<XdgToplevelConfigure *>(lastAcknowledgedConfigure())) {
+            fullscreen = configureEvent->states & XdgToplevelInterface::State::FullScreen;
+        }
+        if (!fullscreen) {
+            const QPointF anchor = interactiveMoveResizeAnchor();
+            const QPointF offset = interactiveMoveOffset();
+            frameGeometry.moveTopLeft(QPointF(anchor.x() - offset.x() * frameGeometry.width(),
+                                              anchor.y() - offset.y() * frameGeometry.height()));
+        }
     }
 
+    maybeUpdateMoveResizeGeometry(frameGeometry);
     updateGeometry(frameGeometry);
 }
 
@@ -239,11 +251,6 @@ void XdgSurfaceWindow::resetHaveNextWindowGeometry()
 
 void XdgSurfaceWindow::moveResizeInternal(const QRectF &rect, MoveResizeMode mode)
 {
-    if (areGeometryUpdatesBlocked()) {
-        setPendingMoveResizeMode(mode);
-        return;
-    }
-
     Q_EMIT frameGeometryAboutToChange();
 
     if (mode != MoveResizeMode::Move) {
@@ -322,38 +329,38 @@ void XdgSurfaceWindow::installPlasmaShellSurface(PlasmaShellSurfaceInterface *sh
         auto moveUnderCursor = [this] {
             if (input()->hasPointer()) {
                 move(input()->globalPointer());
-                keepInArea(workspace()->clientArea(PlacementArea, this));
+                moveResize(keepInArea(moveResizeGeometry(), workspace()->clientArea(PlacementArea, this)));
             }
         };
-        connect(this, &Window::windowShown, this, moveUnderCursor, Qt::SingleShotConnection);
+        connect(this, &Window::readyForPaintingChanged, this, moveUnderCursor, Qt::SingleShotConnection);
     };
     auto updateRole = [this, shellSurface] {
-        NET::WindowType type = NET::Unknown;
+        WindowType type = WindowType::Unknown;
         switch (shellSurface->role()) {
         case PlasmaShellSurfaceInterface::Role::Desktop:
-            type = NET::Desktop;
+            type = WindowType::Desktop;
             break;
         case PlasmaShellSurfaceInterface::Role::Panel:
-            type = NET::Dock;
+            type = WindowType::Dock;
             break;
         case PlasmaShellSurfaceInterface::Role::OnScreenDisplay:
-            type = NET::OnScreenDisplay;
+            type = WindowType::OnScreenDisplay;
             break;
         case PlasmaShellSurfaceInterface::Role::Notification:
-            type = NET::Notification;
+            type = WindowType::Notification;
             break;
         case PlasmaShellSurfaceInterface::Role::ToolTip:
-            type = NET::Tooltip;
+            type = WindowType::Tooltip;
             break;
         case PlasmaShellSurfaceInterface::Role::CriticalNotification:
-            type = NET::CriticalNotification;
+            type = WindowType::CriticalNotification;
             break;
         case PlasmaShellSurfaceInterface::Role::AppletPopup:
-            type = NET::AppletPopup;
+            type = WindowType::AppletPopup;
             break;
         case PlasmaShellSurfaceInterface::Role::Normal:
         default:
-            type = NET::Normal;
+            type = WindowType::Normal;
             break;
         }
         if (m_windowType == type) {
@@ -361,13 +368,13 @@ void XdgSurfaceWindow::installPlasmaShellSurface(PlasmaShellSurfaceInterface *sh
         }
         m_windowType = type;
         switch (m_windowType) {
-        case NET::Desktop:
-        case NET::Dock:
-        case NET::OnScreenDisplay:
-        case NET::Notification:
-        case NET::CriticalNotification:
-        case NET::Tooltip:
-        case NET::AppletPopup:
+        case WindowType::Desktop:
+        case WindowType::Dock:
+        case WindowType::OnScreenDisplay:
+        case WindowType::Notification:
+        case WindowType::CriticalNotification:
+        case WindowType::Tooltip:
+        case WindowType::AppletPopup:
             setOnAllDesktops(true);
 #if KWIN_BUILD_ACTIVITIES
             setOnAllActivities(true);
@@ -408,6 +415,8 @@ XdgToplevelWindow::XdgToplevelWindow(XdgToplevelInterface *shellSurface)
     : XdgSurfaceWindow(shellSurface->xdgSurface())
     , m_shellSurface(shellSurface)
 {
+    setOutput(workspace()->activeOutput());
+    setMoveResizeOutput(workspace()->activeOutput());
     setDesktops({VirtualDesktopManager::self()->currentDesktop()});
 #if KWIN_BUILD_ACTIVITIES
     if (auto a = Workspace::self()->activities()) {
@@ -480,6 +489,9 @@ void XdgToplevelWindow::handleRoleDestroyed()
     if (m_serverDecoration) {
         m_serverDecoration->disconnect(this);
     }
+    if (m_xdgDialog) {
+        m_xdgDialog->disconnect(this);
+    }
 
     m_shellSurface->disconnect(this);
 
@@ -664,6 +676,10 @@ void XdgToplevelWindow::closeWindow()
 
 XdgSurfaceConfigure *XdgToplevelWindow::sendRoleConfigure() const
 {
+    surface()->setPreferredBufferScale(preferredBufferScale());
+    surface()->setPreferredBufferTransform(preferredBufferTransform());
+    surface()->setPreferredColorDescription(preferredColorDescription());
+
     QSize framePadding(0, 0);
     if (m_nextDecoration) {
         framePadding.setWidth(m_nextDecoration->borderLeft() + m_nextDecoration->borderRight());
@@ -723,9 +739,7 @@ void XdgToplevelWindow::doMinimize()
 {
     if (m_isInitialized) {
         if (isMinimized()) {
-            workspace()->windowHidden(this);
-        } else {
-            Q_EMIT windowShown(this);
+            workspace()->activateNextWindow(this);
         }
     }
     workspace()->updateMinimizedOfTransients(this);
@@ -804,9 +818,7 @@ static Qt::Edges anchorsForQuickTileMode(QuickTileMode mode)
 
 void XdgToplevelWindow::doSetQuickTileMode()
 {
-    return; // gtk apps are broken, see https://gitlab.gnome.org/GNOME/gtk/-/merge_requests/6742
-
-    const Qt::Edges anchors = anchorsForQuickTileMode(quickTileMode());
+    const Qt::Edges anchors = anchorsForQuickTileMode(m_requestedQuickTileMode);
 
     if (anchors & Qt::LeftEdge) {
         m_nextStates |= XdgToplevelInterface::State::TiledLeft;
@@ -864,6 +876,36 @@ void XdgToplevelWindow::doSetSuspended()
     scheduleConfigure();
 }
 
+void XdgToplevelWindow::doSetPreferredBufferScale()
+{
+    if (isDeleted()) {
+        return;
+    }
+    if (m_isInitialized) {
+        scheduleConfigure();
+    }
+}
+
+void XdgToplevelWindow::doSetPreferredBufferTransform()
+{
+    if (isDeleted()) {
+        return;
+    }
+    if (m_isInitialized) {
+        scheduleConfigure();
+    }
+}
+
+void XdgToplevelWindow::doSetPreferredColorDescription()
+{
+    if (isDeleted()) {
+        return;
+    }
+    if (m_isInitialized) {
+        scheduleConfigure();
+    }
+}
+
 bool XdgToplevelWindow::takeFocus()
 {
     if (wantsInput()) {
@@ -968,8 +1010,8 @@ void XdgToplevelWindow::handleResizeRequested(SeatInterface *seat, XdgToplevelIn
     } else {
         cursorPos = input()->tablet()->position();
     }
-    setInteractiveMoveOffset(cursorPos - pos()); // map from global
-    setInvertedInteractiveMoveOffset(rect().bottomRight() - interactiveMoveOffset());
+    setInteractiveMoveResizeAnchor(cursorPos);
+    setInteractiveMoveOffset(QPointF((cursorPos.x() - x()) / width(), (cursorPos.y() - y()) / height())); // map from global
     setUnrestrictedInteractiveMoveResize(false);
     Gravity gravity;
     switch (anchor) {
@@ -1026,6 +1068,26 @@ void XdgToplevelWindow::handleStatesAcknowledged(const XdgToplevelInterface::Sta
         updateFullScreenMode(states & XdgToplevelInterface::State::FullScreen);
     }
 
+    if (delta & (XdgToplevelInterface::State::TiledLeft | XdgToplevelInterface::State::TiledTop | XdgToplevelInterface::State::TiledRight | XdgToplevelInterface::State::TiledBottom)) {
+        QuickTileMode newTileMode = QuickTileFlag::None;
+        if (states & XdgToplevelInterface::State::TiledLeft) {
+            newTileMode |= QuickTileFlag::Left;
+        }
+        if (states & XdgToplevelInterface::State::TiledTop) {
+            newTileMode |= QuickTileFlag::Top;
+        }
+        if (states & XdgToplevelInterface::State::TiledRight) {
+            newTileMode |= QuickTileFlag::Right;
+        }
+        if (states & XdgToplevelInterface::State::TiledBottom) {
+            newTileMode |= QuickTileFlag::Bottom;
+        }
+
+        if (newTileMode != quickTileMode()) {
+            setTile(workspace()->tileManager(output())->quickTile(newTileMode));
+        }
+    }
+
     m_acknowledgedStates = states;
 }
 
@@ -1235,7 +1297,6 @@ void XdgToplevelWindow::initialize()
     scheduleConfigure();
     updateColorScheme();
     updateCapabilities();
-    updateClientOutputs();
     setupWindowManagementInterface();
 
     m_isInitialized = true;
@@ -1438,6 +1499,17 @@ void XdgToplevelWindow::installPalette(ServerSideDecorationPaletteInterface *pal
     updateColorScheme();
 }
 
+void XdgToplevelWindow::installXdgDialogV1(XdgDialogV1Interface *dialog)
+{
+    m_xdgDialog = dialog;
+
+    connect(dialog, &XdgDialogV1Interface::modalChanged, this, &Window::setModal);
+    connect(dialog, &QObject::destroyed, this, [this] {
+        setModal(false);
+    });
+    setModal(dialog->isModal());
+}
+
 void XdgToplevelWindow::setFullScreen(bool set)
 {
     if (!isFullScreenable()) {
@@ -1482,7 +1554,7 @@ void XdgToplevelWindow::maximize(MaximizeMode mode)
         return;
     }
 
-    const QRectF clientArea = isElectricBorderMaximizing() ? workspace()->clientArea(MaximizeArea, this, Cursors::self()->mouse()->pos()) : workspace()->clientArea(MaximizeArea, this, moveResizeOutput());
+    const QRectF clientArea = isElectricBorderMaximizing() ? workspace()->clientArea(MaximizeArea, this, interactiveMoveResizeAnchor()) : workspace()->clientArea(MaximizeArea, this, moveResizeOutput());
 
     const MaximizeMode oldMode = m_requestedMaximizeMode;
     const QRectF oldGeometry = moveResizeGeometry();
@@ -1492,6 +1564,7 @@ void XdgToplevelWindow::maximize(MaximizeMode mode)
         return;
     }
 
+    const auto oldQuickTileMode = requestedQuickTileMode();
     Q_EMIT maximizedAboutToChange(mode);
     m_requestedMaximizeMode = mode;
 
@@ -1515,7 +1588,7 @@ void XdgToplevelWindow::maximize(MaximizeMode mode)
         setNoBorder(m_requestedMaximizeMode == MaximizeFull);
     }
 
-    if (quickTileMode() == QuickTileMode(QuickTileFlag::None)) {
+    if (oldQuickTileMode == QuickTileMode(QuickTileFlag::None)) {
         QRectF savedGeometry = geometryRestore();
         if (!(oldMode & MaximizeVertical)) {
             savedGeometry.setTop(oldGeometry.top());
@@ -1566,25 +1639,18 @@ void XdgToplevelWindow::maximize(MaximizeMode mode)
         }
     }
 
-    const auto oldQuickTileMode = quickTileMode();
     if (m_requestedMaximizeMode == MaximizeFull) {
         if (options->electricBorderMaximize()) {
             updateQuickTileMode(QuickTileFlag::Maximize);
         } else {
             updateQuickTileMode(QuickTileFlag::None);
         }
-        setTile(nullptr);
     } else {
         updateQuickTileMode(QuickTileFlag::None);
     }
 
     moveResize(geometry);
 
-    if (oldQuickTileMode != quickTileMode()) {
-        doSetQuickTileMode();
-        Q_EMIT quickTileModeChanged();
-    }
-
     doSetMaximized();
 }
 
@@ -1592,7 +1658,10 @@ XdgPopupWindow::XdgPopupWindow(XdgPopupInterface *shellSurface)
     : XdgSurfaceWindow(shellSurface->xdgSurface())
     , m_shellSurface(shellSurface)
 {
-    m_windowType = NET::Unknown;
+    setOutput(workspace()->activeOutput());
+    setMoveResizeOutput(workspace()->activeOutput());
+
+    m_windowType = WindowType::Unknown;
 
     connect(shellSurface, &XdgPopupInterface::grabRequested,
             this, &XdgPopupWindow::handleGrabRequested);
@@ -1718,6 +1787,10 @@ bool XdgPopupWindow::acceptsFocus() const
 
 XdgSurfaceConfigure *XdgPopupWindow::sendRoleConfigure() const
 {
+    surface()->setPreferredBufferScale(preferredBufferScale());
+    surface()->setPreferredBufferTransform(preferredBufferTransform());
+    surface()->setPreferredColorDescription(preferredColorDescription());
+
     const QPointF parentPosition = transientFor()->framePosToClientPos(transientFor()->pos());
     const QPointF popupPosition = moveResizeGeometry().topLeft() - parentPosition;
 
@@ -1752,6 +1825,36 @@ void XdgPopupWindow::initialize()
     scheduleConfigure();
 }
 
+void XdgPopupWindow::doSetPreferredBufferScale()
+{
+    if (isDeleted()) {
+        return;
+    }
+    if (m_shellSurface->isConfigured()) {
+        scheduleConfigure();
+    }
+}
+
+void XdgPopupWindow::doSetPreferredBufferTransform()
+{
+    if (isDeleted()) {
+        return;
+    }
+    if (m_shellSurface->isConfigured()) {
+        scheduleConfigure();
+    }
+}
+
+void XdgPopupWindow::doSetPreferredColorDescription()
+{
+    if (isDeleted()) {
+        return;
+    }
+    if (m_shellSurface->isConfigured()) {
+        scheduleConfigure();
+    }
+}
+
 } // namespace KWin
 
 #include "moc_xdgshellwindow.cpp"
