@@ -27,6 +27,7 @@ class KillPrompt;
 class PlasmaShellSurfaceInterface;
 class ServerSideDecorationInterface;
 class ServerSideDecorationPaletteInterface;
+class XdgDialogV1Interface;
 class XdgToplevelDecorationV1Interface;
 class Output;
 
@@ -56,7 +57,7 @@ public:
     explicit XdgSurfaceWindow(XdgSurfaceInterface *shellSurface);
     ~XdgSurfaceWindow() override;
 
-    NET::WindowType windowType() const override;
+    WindowType windowType() const override;
     QRectF frameRectToBufferRect(const QRectF &rect) const override;
     void destroyWindow() override;
 
@@ -76,7 +77,7 @@ protected:
 
     QPointer<PlasmaShellSurfaceInterface> m_plasmaShellSurface;
 
-    NET::WindowType m_windowType = NET::Normal;
+    WindowType m_windowType = WindowType::Normal;
     Gravity m_nextGravity = Gravity::None;
 
 private:
@@ -159,6 +160,7 @@ public:
     void installServerDecoration(ServerSideDecorationInterface *decoration);
     void installPalette(ServerSideDecorationPaletteInterface *palette);
     void installXdgDecoration(XdgToplevelDecorationV1Interface *decoration);
+    void installXdgDialogV1(XdgDialogV1Interface *dialog);
 
 protected:
     XdgSurfaceConfigure *sendRoleConfigure() const override;
@@ -175,6 +177,9 @@ protected:
     bool acceptsFocus() const override;
     void doSetQuickTileMode() override;
     void doSetSuspended() override;
+    void doSetPreferredBufferScale() override;
+    void doSetPreferredBufferTransform() override;
+    void doSetPreferredColorDescription() override;
 
 private:
     void handleWindowTitleChanged();
@@ -213,6 +218,7 @@ private:
     QPointer<ServerSideDecorationPaletteInterface> m_paletteInterface;
     QPointer<ServerSideDecorationInterface> m_serverDecoration;
     QPointer<XdgToplevelDecorationV1Interface> m_xdgDecoration;
+    QPointer<XdgDialogV1Interface> m_xdgDialog;
     XdgToplevelInterface *m_shellSurface;
     XdgToplevelInterface::States m_nextStates;
     XdgToplevelInterface::States m_acknowledgedStates;
@@ -257,6 +263,9 @@ protected:
     bool acceptsFocus() const override;
     XdgSurfaceConfigure *sendRoleConfigure() const override;
     void handleRoleDestroyed() override;
+    void doSetPreferredBufferScale() override;
+    void doSetPreferredBufferTransform() override;
+    void doSetPreferredColorDescription() override;
 
 private:
     void handleGrabRequested(SeatInterface *seat, quint32 serial);
